@@ -1,6 +1,6 @@
 /* FAudio# - C# Wrapper for FAudio
  *
- * Copyright (c) 2018 Ethan Lee.
+ * Copyright (c) 2020 Ethan Lee.
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -59,8 +59,8 @@ public static class FAudio
 	public const uint FAUDIO_TARGET_VERSION = 8;
 
 	public const uint FAUDIO_ABI_VERSION =		 0;
-	public const uint FAUDIO_MAJOR_VERSION =	19;
-	public const uint FAUDIO_MINOR_VERSION =	12;
+	public const uint FAUDIO_MAJOR_VERSION =	20;
+	public const uint FAUDIO_MINOR_VERSION =	 4;
 	public const uint FAUDIO_PATCH_VERSION =	 0;
 
 	public const uint FAUDIO_COMPILED_VERSION = (
@@ -2154,9 +2154,21 @@ public static class FAudio
 		public int max_frame_size;
 	}
 
+	[StructLayout(LayoutKind.Sequential)]
+	public struct stb_vorbis_comment
+	{
+		public IntPtr vendor;
+
+		public int comment_list_length;
+		public IntPtr comment_list;
+	}
+
 	[DllImport(nativeLibName, CallingConvention = CallingConvention.Cdecl)]
 	public static extern stb_vorbis_info stb_vorbis_get_info(IntPtr f);
 
+	[DllImport(nativeLibName, CallingConvention = CallingConvention.Cdecl)]
+	public static extern stb_vorbis_comment stb_vorbis_get_comment(IntPtr f);
+
 	[DllImport(nativeLibName, CallingConvention = CallingConvention.Cdecl)]
 	public static extern int stb_vorbis_get_error(IntPtr f);
 
