@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2018 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2020 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -34,6 +34,29 @@
 #include "stb.h"
 #define FACT_INTERNAL_rng() ((float) stb_frand())
 
+/* XACT Versions */
+
+#define FACT_CONTENT_VERSION_3_4 45
+#define FACT_CONTENT_VERSION_3_1 44
+
+static inline int FACT_INTERNAL_SupportedContent(uint16_t version)
+{
+	return (	version == FACT_CONTENT_VERSION ||
+			version == FACT_CONTENT_VERSION_3_4 ||
+			version == FACT_CONTENT_VERSION_3_1	);
+}
+
+#define WAVEBANK_HEADER_VERSION		44
+#define WAVEBANK_HEADER_VERSION_3_4	43
+#define WAVEBANK_HEADER_VERSION_3_1	42
+
+static inline int FACT_INTERNAL_SupportedWBContent(uint16_t version)
+{
+	return (	version == WAVEBANK_HEADER_VERSION ||
+			version == WAVEBANK_HEADER_VERSION_3_4 ||
+			version == WAVEBANK_HEADER_VERSION_3_1	);
+}
+
 /* Helper Functions */
 
 #define SWAP_16(x) \
@@ -1954,7 +1977,7 @@ uint32_t FACT_INTERNAL_ParseAudioEngine(
 			dspPresetOffset,
 			dspParameterOffset;
 	uint16_t blob1Count, blob2Count;
-	uint8_t version;
+	uint8_t version, tool;
 	size_t memsize;
 	uint16_t i, j;
 
@@ -1963,13 +1986,23 @@ uint32_t FACT_INTERNAL_ParseAudioEngine(
 	
 	uint8_t se = 0; /* Swap Endian */
 	uint32_t magic = read_u32(&ptr, 0);
-	if (	(magic != 0x46534758 && !(se = (magic == 0x58475346))) /* 'XGSF' */ ||
-		read_u16(&ptr, se) != FACT_CONTENT_VERSION ||
-		read_u16(&ptr, se) != 42 /* Tool version */	)
+	se = magic == 0x58475346;
+	if (magic != 0x46534758 && magic != 0x58475346) /* 'XGSF' */
 	{
 		return -1; /* TODO: NOT XACT FILE */
 	}
 
+	if (!FACT_INTERNAL_SupportedContent(read_u16(&ptr, se)))
+	{
+		return -2;
+	}
+
+	tool = read_u16(&ptr, se); /* Tool version */
+	if (tool != 42)
+	{
+		return -3;
+	}
+
 	ptr += 2; /* Unknown value */
 
 	/* Last modified, unused */
@@ -1980,7 +2013,7 @@ uint32_t FACT_INTERNAL_ParseAudioEngine(
 	if (	version != 3 &&
 		version != 7	)
 	{
-		return -1; /* TODO: VERSION TOO OLD */
+		return -4; /* TODO: VERSION TOO OLD */
 	}
 
 	/* Object counts */
@@ -2160,6 +2193,13 @@ uint32_t FACT_INTERNAL_ParseAudioEngine(
 		ptr += memsize;
 	}
 
+	/* Peristent Notifications */
+	pEngine->notifications = 0;
+	pEngine->cue_context = NULL;
+	pEngine->sb_context = NULL;
+	pEngine->wb_context = NULL;
+	pEngine->wave_context = NULL;
+
 	/* Finally. */
 	FAudio_assert((ptr - start) == pParams->globalSettingsBufferSize);
 	return 0;
@@ -2399,21 +2439,31 @@ uint32_t FACT_INTERNAL_ParseSoundBank(
 		soundOffset;
 	uint8_t platform;
 	size_t memsize;
-	uint16_t i, j, cur;
+	uint16_t i, j, cur, tool;
 	uint8_t *ptrBookmark;
 
 	uint8_t *ptr = (uint8_t*) pvBuffer;
 	uint8_t *start = ptr;
 
 	uint32_t magic = read_u32(&ptr, 0);
-	uint8_t se = 0; /* Swap Endian */
-	if (	(magic != 0x4B424453 && !(se = (magic == 0x5344424B))) /* 'SDBK' */ ||
-		read_u16(&ptr, se) != FACT_CONTENT_VERSION ||
-		read_u16(&ptr, se) != 43 /* Tool version */	)
+	uint8_t se = magic == 0x5344424B; /* Swap Endian */
+
+	if (magic != 0x4B424453 && magic != 0x5344424B)  /* 'SDBK' */
 	{
 		return -1; /* TODO: NOT XACT FILE */
 	}
 
+	if (!FACT_INTERNAL_SupportedContent(read_u16(&ptr, se)))
+	{
+		return -2;
+	}
+
+	tool = read_u16(&ptr, se); /* Tool version */
+	if (tool != 43)
+	{
+		return -3;
+	}
+
 	/* CRC, unused */
 	ptr += 2;
 
@@ -2432,6 +2482,7 @@ uint32_t FACT_INTERNAL_ParseSoundBank(
 	sb->parentEngine = pEngine;
 	sb->cueList = NULL;
 	sb->notifyOnDestroy = 0;
+	sb->usercontext = NULL;
 
 	cueSimpleCount = read_u16(&ptr, se);
 	cueComplexCount = read_u16(&ptr, se);
@@ -2925,13 +2976,21 @@ uint32_t FACT_INTERNAL_ParseWaveBank(
 			DOSWAP_32(header.Segments[i].dwLength);
 		}
 	}
-	if (	header.dwSignature != 0x444E4257 ||
-		header.dwVersion != FACT_CONTENT_VERSION ||
-		header.dwHeaderVersion != 44	)
+	if (header.dwSignature != 0x444E4257)
 	{
 		return -1; /* TODO: NOT XACT FILE */
 	}
 
+	if (!FACT_INTERNAL_SupportedContent(header.dwVersion))
+	{
+		return -2;
+	}
+
+	if (!FACT_INTERNAL_SupportedWBContent(header.dwHeaderVersion))
+	{
+		return -3;
+	}
+
 	wb = (FACTWaveBank*) pEngine->pMalloc(sizeof(FACTWaveBank));
 	wb->parentEngine = pEngine;
 	wb->waveList = NULL;
@@ -2939,6 +2998,7 @@ uint32_t FACT_INTERNAL_ParseWaveBank(
 	wb->packetSize = packetSize;
 	wb->io = io;
 	wb->notifyOnDestroy = 0;
+	wb->usercontext = NULL;
 
 	/* WaveBank Data */
 	SEEKSET(header.Segments[FACT_WAVEBANK_SEGIDX_BANKDATA].dwOffset)
