@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2018 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2020 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -49,8 +49,21 @@ MAKE_SUBFORMAT_GUID(IEEE_FLOAT, 3);
 MAKE_SUBFORMAT_GUID(XMAUDIO2, FAUDIO_FORMAT_XMAUDIO2);
 MAKE_SUBFORMAT_GUID(WMAUDIO2, FAUDIO_FORMAT_WMAUDIO2);
 MAKE_SUBFORMAT_GUID(WMAUDIO3, FAUDIO_FORMAT_WMAUDIO3);
+MAKE_SUBFORMAT_GUID(WMAUDIO_LOSSLESS, FAUDIO_FORMAT_WMAUDIO_LOSSLESS);
 #undef MAKE_SUBFORMAT_GUID
 
+#ifdef FAUDIO_DUMP_VOICES
+static void FAudio_DUMPVOICE_Init(const FAudioSourceVoice *voice);
+static void FAudio_DUMPVOICE_Finalize(const FAudioSourceVoice *voice);
+static void FAudio_DUMPVOICE_WriteBuffer(
+	const FAudioSourceVoice *voice,
+	const FAudioBuffer *pBuffer,
+	const FAudioBufferWMA *pBufferWMA,
+	const uint32_t playBegin,
+	const uint32_t playLength
+);
+#endif /* FAUDIO_DUMP_VOICES */
+
 /* FAudio Version */
 
 uint32_t FAudioLinkedVersion(void)
@@ -200,6 +213,8 @@ uint32_t FAudio_Initialize(
 	FAudio_assert(Flags == 0);
 	FAudio_assert(XAudio2Processor == FAUDIO_DEFAULT_PROCESSOR);
 
+	audio->initFlags = Flags;
+
 	/* FIXME: This is lazy... */
 	audio->decodeCache = (float*) audio->pMalloc(sizeof(float));
 	audio->resampleCache = (float*) audio->pMalloc(sizeof(float));
@@ -312,6 +327,34 @@ uint32_t FAudio_CreateSourceVoice(
 		}
 		(*ppSourceVoice)->src.format = &fmtex->Format;
 	}
+	else if (pSourceFormat->wFormatTag == FAUDIO_FORMAT_MSADPCM)
+	{
+		FAudioADPCMWaveFormat *fmtex = (FAudioADPCMWaveFormat*) audio->pMalloc(
+			sizeof(FAudioADPCMWaveFormat)
+		);
+
+		/* Copy what we can, ideally the sizes match! */
+		size_t cbSize = sizeof(FAudioWaveFormatEx) + pSourceFormat->cbSize;
+		FAudio_memcpy(
+			fmtex,
+			pSourceFormat,
+			FAudio_min(cbSize, sizeof(FAudioADPCMWaveFormat))
+		);
+		if (cbSize < sizeof(FAudioADPCMWaveFormat))
+		{
+			FAudio_zero(
+				((uint8_t*) fmtex) + cbSize,
+				sizeof(FAudioADPCMWaveFormat) - cbSize
+			);
+		}
+
+		/* XAudio2 does not validate this input! */
+		fmtex->wfx.cbSize = sizeof(FAudioADPCMWaveFormat) - sizeof(FAudioWaveFormatEx);
+		fmtex->wSamplesPerBlock = ((
+			fmtex->wfx.nBlockAlign / fmtex->wfx.nChannels
+		) - 6) * 2;
+		(*ppSourceVoice)->src.format = &fmtex->wfx;
+	}
 	else
 	{
 		/* direct copy anything else */
@@ -335,7 +378,7 @@ uint32_t FAudio_CreateSourceVoice(
 
 	if ((*ppSourceVoice)->src.format->wFormatTag == FAUDIO_FORMAT_EXTENSIBLE)
 	{
-		FAudioWaveFormatExtensible *fmtex = (FAudioWaveFormatExtensible*)(*ppSourceVoice)->src.format;
+		FAudioWaveFormatExtensible *fmtex = (FAudioWaveFormatExtensible*) (*ppSourceVoice)->src.format;
 
 		#define COMPARE_GUID(type) \
 			(FAudio_memcmp( \
@@ -385,6 +428,7 @@ uint32_t FAudio_CreateSourceVoice(
 		}
 		else if (	COMPARE_GUID(WMAUDIO2) ||
 				COMPARE_GUID(WMAUDIO3) ||
+				COMPARE_GUID(WMAUDIO_LOSSLESS) ||
 				COMPARE_GUID(XMAUDIO2)	)
 		{
 #ifdef HAVE_FFMPEG
@@ -430,6 +474,7 @@ uint32_t FAudio_CreateSourceVoice(
 	(*ppSourceVoice)->src.curBufferOffset = 0;
 
 	/* Sends/Effects */
+	FAudio_INTERNAL_VoiceOutputFrequency(*ppSourceVoice, pSendList);
 	FAudioVoice_SetEffectChain(*ppSourceVoice, pEffectChain);
 	FAudioVoice_SetOutputVoices(*ppSourceVoice, pSendList);
 
@@ -478,6 +523,10 @@ uint32_t FAudio_CreateSourceVoice(
 	);
 	FAudio_AddRef(audio);
 
+#ifdef FAUDIO_DUMP_VOICES
+	FAudio_DUMPVOICE_Init(*ppSourceVoice);
+#endif /* FAUDIO_DUMP_VOICES */
+
 	LOG_API_EXIT(audio)
 	return 0;
 }
@@ -518,6 +567,20 @@ uint32_t FAudio_CreateSubmixVoice(
 	(*ppSubmixVoice)->mix.inputSampleRate = InputSampleRate;
 	(*ppSubmixVoice)->mix.processingStage = ProcessingStage;
 
+	/* Resampler */
+	if (InputChannels == 1)
+	{
+		(*ppSubmixVoice)->mix.resample = FAudio_INTERNAL_ResampleMono;
+	}
+	else if (InputChannels == 2)
+	{
+		(*ppSubmixVoice)->mix.resample = FAudio_INTERNAL_ResampleStereo;
+	}
+	else
+	{
+		(*ppSubmixVoice)->mix.resample = FAudio_INTERNAL_ResampleGeneric;
+	}
+
 	/* Sample Storage */
 	(*ppSubmixVoice)->mix.inputSamples = ((uint32_t) FAudio_ceil(
 		audio->updateSize *
@@ -533,6 +596,7 @@ uint32_t FAudio_CreateSubmixVoice(
 	);
 
 	/* Sends/Effects */
+	FAudio_INTERNAL_VoiceOutputFrequency(*ppSubmixVoice, pSendList);
 	FAudioVoice_SetEffectChain(*ppSubmixVoice, pEffectChain);
 	FAudioVoice_SetOutputVoices(*ppSubmixVoice, pSendList);
 
@@ -617,10 +681,42 @@ uint32_t FAudio_CreateMasteringVoice(
 	FAudio_zero(&(*ppMasteringVoice)->sends, sizeof(FAudioVoiceSends));
 	FAudioVoice_SetEffectChain(*ppMasteringVoice, pEffectChain);
 
-	/* Platform Device */
+	/* This is now safe enough to assign */
 	audio->master = *ppMasteringVoice;
+
+	/* Build the device format.
+	 * The most unintuitive part of this is the use of outputChannels
+	 * instead of master.inputChannels. Bizarrely, the effect chain can
+	 * dictate the _actual_ output channel count, and when the channel count
+	 * mismatches, we have to add a staging buffer for effects to process on
+	 * before ultimately copying the final result to the device. ARGH.
+	 */
+	WriteWaveFormatExtensible(
+		&audio->mixFormat,
+		audio->master->outputChannels,
+		audio->master->master.inputSampleRate
+	);
+
+	/* Platform Device */
 	FAudio_AddRef(audio);
-	FAudio_PlatformInit(audio, DeviceIndex);
+	FAudio_PlatformInit(
+		audio,
+		audio->initFlags,
+		DeviceIndex,
+		&audio->mixFormat,
+		&audio->updateSize,
+		&audio->platform
+	);
+	if (audio->platform == NULL)
+	{
+		FAudioVoice_DestroyVoice(*ppMasteringVoice);
+		*ppMasteringVoice = NULL;
+
+		/* Not the best code, but it's probably true? */
+		return FAUDIO_E_DEVICE_INVALIDATED;
+	}
+	audio->master->outputChannels = audio->mixFormat.Format.nChannels;
+	audio->master->master.inputSampleRate = audio->mixFormat.Format.nSamplesPerSec;
 
 	/* Effect Chain Cache */
 	if ((*ppMasteringVoice)->master.inputChannels != (*ppMasteringVoice)->outputChannels)
@@ -866,6 +962,22 @@ void FAudio_SetDebugConfiguration(
 #endif /* FAUDIO_DISABLE_DEBUGCONFIGURATION */
 }
 
+void FAudio_GetProcessingQuantum(
+	FAudio *audio,
+	uint32_t *quantumNumerator,
+	uint32_t *quantumDenominator
+) {
+	FAudio_assert(audio->master != NULL);
+	if (quantumNumerator != NULL)
+	{
+		*quantumNumerator = audio->updateSize;
+	}
+	if (quantumDenominator != NULL)
+	{
+		*quantumDenominator = audio->master->master.inputSampleRate;
+	}
+}
+
 /* FAudioVoice Interface */
 
 void FAudioVoice_GetVoiceDetails(
@@ -905,10 +1017,6 @@ uint32_t FAudioVoice_SetOutputVoices(
 ) {
 	uint32_t i;
 	uint32_t outChannels;
-	uint32_t channelCount = 0;
-	uint32_t outSampleRate;
-	uint32_t newResampleSamples;
-	uint64_t resampleSanityCheck;
 	FAudioVoiceSends defaultSends;
 	FAudioSendDescriptor defaultSend;
 
@@ -923,6 +1031,19 @@ uint32_t FAudioVoice_SetOutputVoices(
 	FAudio_PlatformLockMutex(voice->sendLock);
 	LOG_MUTEX_LOCK(voice->audio, voice->sendLock)
 
+	if (FAudio_INTERNAL_VoiceOutputFrequency(voice, pSendList) != 0)
+	{
+		LOG_ERROR(
+			voice->audio,
+			"%s",
+			"Changing the sample rate while an effect chain is attached is invalid!"
+		)
+		FAudio_PlatformUnlockMutex(voice->sendLock);
+		LOG_MUTEX_UNLOCK(voice->audio, voice->sendLock)
+		LOG_API_EXIT(voice->audio)
+		return FAUDIO_E_INVALID_CALL;
+	}
+
 	/* FIXME: This is lazy... */
 	for (i = 0; i < voice->sends.SendCount; i += 1)
 	{
@@ -972,12 +1093,10 @@ uint32_t FAudioVoice_SetOutputVoices(
 		/* No sends? Nothing to do... */
 		voice->sendCoefficients = NULL;
 		voice->sendMix = NULL;
-		voice->sendFilter = NULL;
-		voice->sendFilterState = NULL;
 		FAudio_zero(&voice->sends, sizeof(FAudioVoiceSends));
+
 		FAudio_PlatformUnlockMutex(voice->sendLock);
 		LOG_MUTEX_UNLOCK(voice->audio, voice->sendLock)
-
 		LOG_API_EXIT(voice->audio)
 		return 0;
 	}
@@ -1107,73 +1226,6 @@ uint32_t FAudioVoice_SetOutputVoices(
 		}
 	}
 
-	/* Allocate resample cache */
-	outSampleRate = voice->sends.pSends[0].pOutputVoice->type == FAUDIO_VOICE_MASTER ?
-		voice->sends.pSends[0].pOutputVoice->master.inputSampleRate :
-		voice->sends.pSends[0].pOutputVoice->mix.inputSampleRate;
-	newResampleSamples = (uint32_t) FAudio_ceil(
-		voice->audio->updateSize *
-		(double) outSampleRate /
-		(double) voice->audio->master->master.inputSampleRate
-	);
-	if (voice->type == FAUDIO_VOICE_SOURCE)
-	{
-		channelCount = voice->src.format->nChannels;
-	}
-	else if (voice->type == FAUDIO_VOICE_SUBMIX)
-	{
-		channelCount = voice->mix.inputChannels;
-	}
-	else if (voice->type == FAUDIO_VOICE_MASTER)
-	{
-		channelCount = voice->master.inputChannels;
-	}
-	FAudio_INTERNAL_ResizeResampleCache(
-		voice->audio,
-		newResampleSamples * channelCount
-	);
-	if (voice->type == FAUDIO_VOICE_SOURCE)
-	{
-		voice->src.resampleSamples = newResampleSamples;
-	}
-	else
-	{
-		voice->mix.outputSamples = newResampleSamples;
-
-		voice->mix.resampleStep = DOUBLE_TO_FIXED((
-			(double) voice->mix.inputSampleRate /
-			(double) outSampleRate
-		));
-
-		/* Because we used ceil earlier, there's a chance that
-		 * downsampling submixes will go past the number of samples
-		 * available. Sources can do this thanks to padding, but we
-		 * don't have that luxury for submixes, so unfortunately we
-		 * just have to undo the ceil and turn it into a floor.
-		 * -flibit
-		 */
-		resampleSanityCheck = (
-			voice->mix.resampleStep * voice->mix.outputSamples
-		) >> FIXED_PRECISION;
-		if (resampleSanityCheck > (voice->mix.inputSamples / voice->mix.inputChannels))
-		{
-			voice->mix.outputSamples -= 1;
-		}
-
-		if (voice->mix.inputChannels == 1)
-		{
-			voice->mix.resample = FAudio_INTERNAL_ResampleMono;
-		}
-		else if (voice->mix.inputChannels == 2)
-		{
-			voice->mix.resample = FAudio_INTERNAL_ResampleStereo;
-		}
-		else
-		{
-			voice->mix.resample = FAudio_INTERNAL_ResampleGeneric;
-		}
-	}
-
 	FAudio_PlatformUnlockMutex(voice->sendLock);
 	LOG_MUTEX_UNLOCK(voice->audio, voice->sendLock)
 	LOG_API_EXIT(voice->audio)
@@ -1596,7 +1648,7 @@ uint32_t FAudioVoice_SetOutputFilterParameters(
 	/* Find the send index */
 	if (pDestinationVoice == NULL && voice->sends.SendCount == 1)
 	{
-		pDestinationVoice = voice->audio->master;
+		pDestinationVoice = voice->sends.pSends[0].pOutputVoice;
 	}
 	for (i = 0; i < voice->sends.SendCount; i += 1)
 	{
@@ -1664,7 +1716,7 @@ void FAudioVoice_GetOutputFilterParameters(
 	/* Find the send index */
 	if (pDestinationVoice == NULL && voice->sends.SendCount == 1)
 	{
-		pDestinationVoice = voice->audio->master;
+		pDestinationVoice = voice->sends.pSends[0].pOutputVoice;
 	}
 	for (i = 0; i < voice->sends.SendCount; i += 1)
 	{
@@ -1843,7 +1895,7 @@ uint32_t FAudioVoice_SetOutputMatrix(
 	/* Find the send index */
 	if (pDestinationVoice == NULL && voice->sends.SendCount == 1)
 	{
-		pDestinationVoice = voice->audio->master;
+		pDestinationVoice = voice->sends.pSends[0].pOutputVoice;
 	}
 	FAudio_assert(pDestinationVoice != NULL);
 	for (i = 0; i < voice->sends.SendCount; i += 1)
@@ -1969,6 +2021,10 @@ void FAudioVoice_DestroyVoice(FAudioVoice *voice)
 	{
 		FAudioBufferEntry *entry, *next;
 
+#ifdef FAUDIO_DUMP_VOICES
+		FAudio_DUMPVOICE_Finalize((FAudioSourceVoice*) voice);
+#endif /* FAUDIO_DUMP_VOICES */
+
 		FAudio_PlatformLockMutex(voice->audio->sourceLock);
 		LOG_MUTEX_LOCK(voice->audio, voice->audio->sourceLock)
 		while (voice == voice->audio->processingSource)
@@ -2020,7 +2076,15 @@ void FAudioVoice_DestroyVoice(FAudioVoice *voice)
 	}
 	else if (voice->type == FAUDIO_VOICE_MASTER)
 	{
-		FAudio_PlatformQuit(voice->audio);
+		if (voice->audio->platform != NULL)
+		{
+			FAudio_PlatformQuit(voice->audio->platform);
+			voice->audio->platform = NULL;
+		}
+		if (voice->master.effectCache != NULL)
+		{
+			voice->audio->pFree(voice->master.effectCache);
+		}
 		voice->audio->master = NULL;
 	}
 
@@ -2218,10 +2282,11 @@ uint32_t FAudioSourceVoice_SubmitSourceBuffer(
 	{
 		if (voice->src.format->wFormatTag == FAUDIO_FORMAT_MSADPCM)
 		{
+			FAudioADPCMWaveFormat *fmtex = (FAudioADPCMWaveFormat*) voice->src.format;
 			playLength = (
 				pBuffer->AudioBytes /
-				voice->src.format->nBlockAlign *
-				(((voice->src.format->nBlockAlign / voice->src.format->nChannels) - 6) * 2)
+				fmtex->wfx.nBlockAlign *
+				fmtex->wSamplesPerBlock
 			) - playBegin;
 		}
 		else if (pBufferWMA != NULL)
@@ -2270,7 +2335,7 @@ uint32_t FAudioSourceVoice_SubmitSourceBuffer(
 	/* For ADPCM, round down to the nearest sample block size */
 	if (voice->src.format->wFormatTag == FAUDIO_FORMAT_MSADPCM)
 	{
-		adpcmMask = ((voice->src.format->nBlockAlign / voice->src.format->nChannels) - 6) * 2;
+		adpcmMask = ((FAudioADPCMWaveFormat*) voice->src.format)->wSamplesPerBlock;
 		playBegin -= playBegin % adpcmMask;
 		playLength -= playLength % adpcmMask;
 		loopBegin -= loopBegin % adpcmMask;
@@ -2303,6 +2368,14 @@ uint32_t FAudioSourceVoice_SubmitSourceBuffer(
 		entry->buffer.LoopCount = 0;
 	}
 
+#ifdef FAUDIO_DUMP_VOICES
+	/* dumping current buffer, append into "data" section */
+	if (pBuffer->pAudioData != NULL && playLength > 0)
+	{
+		FAudio_DUMPVOICE_WriteBuffer(voice, pBuffer, pBufferWMA, playBegin, playLength);
+	}
+#endif /* FAUDIO_DUMP_VOICES */
+
 	/* Submit! */
 	FAudio_PlatformLockMutex(voice->src.bufferLock);
 	LOG_MUTEX_LOCK(voice->audio, voice->src.bufferLock)
@@ -2611,12 +2684,307 @@ FAUDIOAPI uint32_t FAudioMasteringVoice_GetChannelMask(
 ) {
 	LOG_API_ENTER(voice->audio)
 	FAudio_assert(voice->type == FAUDIO_VOICE_MASTER);
-	FAudio_assert(voice->audio->mixFormat != NULL);
 	FAudio_assert(pChannelMask != NULL);
 
-	*pChannelMask = voice->audio->mixFormat->dwChannelMask;
+	*pChannelMask = voice->audio->mixFormat.dwChannelMask;
 	LOG_API_EXIT(voice->audio)
 	return 0;
 }
 
+#ifdef FAUDIO_DUMP_VOICES
+
+static inline FAudioIOStreamOut *DumpVoices_fopen(
+	const FAudioSourceVoice *voice,
+	const FAudioWaveFormatEx *format,
+	const char *mode,
+	const char *ext
+) {
+	char loc[64];
+	uint16_t format_tag = format->wFormatTag;
+	uint16_t format_ex_tag = 0;
+	if (format->wFormatTag == FAUDIO_FORMAT_EXTENSIBLE)
+	{
+		/* get the GUID of the extended subformat */
+		const FAudioWaveFormatExtensible *format_ex =
+				(const FAudioWaveFormatExtensible*) format;
+		format_ex_tag = (uint16_t) (format_ex->SubFormat.Data1);
+	}
+	FAudio_snprintf(
+		loc,
+		sizeof(loc),
+		"FA_fmt_0x%04X_0x%04X_0x%016lX%s.wav",
+		format_tag,
+		format_ex_tag,
+		(uint64_t) voice,
+		ext
+	);
+	FAudioIOStreamOut *fileOut = FAudio_fopen_out(loc, mode);
+	return fileOut;
+}
+
+static inline void DumpVoices_finalize_section(
+	const FAudioSourceVoice *voice,
+	const FAudioWaveFormatEx *format,
+	const char *section /* one of "data" or "dpds" */
+) {
+	/* data file only contains the real data bytes */
+	FAudioIOStreamOut *io_data = DumpVoices_fopen(voice, format, "rb", section);
+	if (!io_data)
+	{
+		return;
+	}
+	FAudio_PlatformLockMutex((FAudioMutex) io_data->lock);
+	size_t file_size_data = io_data->size(io_data->data);
+	if (file_size_data == 0)
+	{
+		/* nothing to do */
+		/* close data file */
+		FAudio_PlatformUnlockMutex((FAudioMutex) io_data->lock);
+		FAudio_close_out(io_data);
+		return;
+	}
+
+	/* we got some data: append data section to main file */
+	FAudioIOStreamOut *io = DumpVoices_fopen(voice, format, "ab", "");
+	if (!io)
+	{
+		/* close data file */
+		FAudio_PlatformUnlockMutex((FAudioMutex) io_data->lock);
+		FAudio_close_out(io_data);
+		return;
+	}
+
+	/* data sub-chunk - 8 bytes + data */
+	/* SubChunk2ID - 4 --> "data" or "dpds" */
+	io->write(io->data, section, 4, 1);
+	/* Subchunk2Size - 4 */
+	uint32_t chunk_size = (uint32_t)file_size_data;
+	io->write(io->data, &chunk_size, 4, 1);
+	/* data */
+	/* fill in data bytes */
+	uint8_t buffer[1024*1024];
+	size_t count;
+	while((count = io_data->read(io_data->data, (void*) buffer, 1, 1024*1024)) > 0)
+	{
+		io->write(io->data, (void*) buffer, 1, count);
+	}
+
+	/* close data file */
+	FAudio_PlatformUnlockMutex((FAudioMutex) io_data->lock);
+	FAudio_close_out(io_data);
+	/* close main file */
+	FAudio_PlatformUnlockMutex((FAudioMutex) io->lock);
+	FAudio_close_out(io);
+}
+
+static void FAudio_DUMPVOICE_Init(const FAudioSourceVoice *voice)
+{
+	const FAudioWaveFormatEx *format = voice->src.format;
+
+	FAudioIOStreamOut *io = DumpVoices_fopen(voice, format, "wb", "");
+	if (!io)
+	{
+		return;
+	}
+	FAudio_PlatformLockMutex((FAudioMutex) io->lock);
+	/* another GREAT ressource
+	 * https://wiki.multimedia.cx/index.php/Microsoft_xWMA
+	 */
+
+
+	/* wave file format taken from
+	 * http://soundfile.sapp.org/doc/WaveFormat
+	 * https://sites.google.com/site/musicgapi/technical-documents/wav-file-format
+	 * |52 49|46 46|52 4A|02 00|
+	 * |c1 sz|af|nc|sp rt|bt rt|
+	 * |ba|bs|da ta|c2 sz|
+
+	 * | R  I  F  F  |chunk size  |W  A  V  E  |f  m  t     |
+	 *                19026
+	 * | 52 49 46 46  52 4A 02 00  57 41 56 45  66 6D 74 20 | RIFFRJ..WAVEfmt
+
+	 * | subchnk size|fmt  |nChan |samplerate  |byte rate   |
+	 * | 50          | 2   |2     |11025       |11289       |
+	 * | 32 00 00 00  02 00 02 00  11 2B 00 00  19 2C 00 00 | 2........+...,..
+
+	 * |blkaln|bps   |efmt |XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX|
+	 * | 512  |4     |32   |500   |7    |256   |0    |512   |
+	 * | 512  |4     |32   |459252      |256         |
+	 * | 00 02|04 00  20 00 F4 01  07 00 00 01  00 00 00 02 | .... .ô.........
+
+	 * | XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX |
+	 * |
+	 * | 00 FF 00 00  00 00 C0 00  40 00 F0 00  00 00 CC 01 | .ÿ....À.@.ð...Ì.
+
+	 * | XXXXXXXXXXXXXXXXXX|d  a   t  a |chunk size  |XXXXX |
+	 * |                   |            |18944       |      |
+	 * | 30 FF 88 01  18 FF 64 61  74 61 00 4A  02 00 00 00 | 0ÿ...ÿdata.J....
+	 */
+
+	uint16_t cbSize = format->cbSize;
+	const char *formatFourcc = "WAVE";
+	uint16_t wFormatTag = format->wFormatTag;
+	/* special handling for WMAUDIO2 */
+	if (wFormatTag == FAUDIO_FORMAT_EXTENSIBLE && cbSize >= 22)
+	{
+		const FAudioWaveFormatExtensible *format_ex =
+				(const FAudioWaveFormatExtensible*) format;
+		uint16_t format_ex_tag = (uint16_t) (format_ex->SubFormat.Data1);
+		if (format_ex_tag == FAUDIO_FORMAT_WMAUDIO2)
+		{
+			cbSize = 0;
+			formatFourcc = "XWMA";
+			wFormatTag = FAUDIO_FORMAT_WMAUDIO2;
+		}
+	}
+
+	{ /* RIFF chunk descriptor - 12 byte */
+		/* ChunkID - 4 */
+		io->write(io->data, "RIFF", 4, 1);
+		/* ChunkSize - 4 */
+		uint32_t filesize = 0; /* the real file size is written in finalize step */
+		io->write(io->data, &filesize, 4, 1);
+		/* Format - 4 */
+		io->write(io->data, formatFourcc, 4, 1);
+	}
+	{ /* fmt sub-chunk 24 */
+		/* Subchunk1ID - 4 */
+		io->write(io->data, "fmt ", 4, 1);
+		/* Subchunk1Size - 4 */
+		/* 18 byte for WAVEFORMATEX and cbSize for WAVEFORMATEXTENDED */
+		uint32_t chunk_data_size = 18 + (uint32_t) cbSize;
+		io->write(io->data, &chunk_data_size, 4, 1);
+		/* AudioFormat - 2 */
+		io->write(io->data, &wFormatTag, 2, 1);
+		/* NumChannels - 2 */
+		io->write(io->data, &format->nChannels, 2, 1);
+		/* SampleRate - 4 */
+		io->write(io->data, &format->nSamplesPerSec, 4, 1);
+		/* ByteRate - 4 */
+		/* SampleRate * NumChannels * BitsPerSample/8 */
+		io->write(io->data, &format->nAvgBytesPerSec, 4, 1);
+		/* BlockAlign - 2 */
+		/* NumChannels * BitsPerSample/8 */
+		io->write(io->data, &format->nBlockAlign, 2, 1);
+		/* BitsPerSample - 2 */
+		io->write(io->data, &format->wBitsPerSample, 2, 1);
+	}
+	/* in case of extensible audio format write the additional data to the file */
+	{
+		/* always write the cbSize */
+		io->write(io->data, &cbSize, 2, 1);
+
+		if (cbSize >= 22)
+		{
+			/* we have a WAVEFORMATEXTENSIBLE struct to write */
+			const FAudioWaveFormatExtensible *format_ex =
+					(const FAudioWaveFormatExtensible*) format;
+			io->write(io->data, &format_ex->Samples.wValidBitsPerSample, 2, 1);
+			io->write(io->data, &format_ex->dwChannelMask,   4, 1);
+			/* write FAudioGUID */
+			io->write(io->data, &format_ex->SubFormat.Data1, 4, 1);
+			io->write(io->data, &format_ex->SubFormat.Data2, 2, 1);
+			io->write(io->data, &format_ex->SubFormat.Data3, 2, 1);
+			io->write(io->data, &format_ex->SubFormat.Data4, 1, 8);
+		}
+		if (format->cbSize > 22)
+		{
+			/* fill up the remaining cbSize bytes with zeros */
+			uint8_t zero = 0;
+			for (uint16_t i=23; i<=format->cbSize; i++)
+			{
+				io->write(io->data, &zero, 1, 1);
+			}
+		}
+	}
+	{ /* dpds sub-chunk - optional - 8 bytes + bufferWMA uint32_t samples */
+		/* create file to hold the bufferWMA samples */
+		FAudioIOStreamOut *io_dpds = DumpVoices_fopen(voice, format, "wb", "dpds");
+		FAudio_close_out(io_dpds);
+		/* io_dpds file will be filled by SubmitBuffer */
+	}
+	{ /* data sub-chunk - 8 bytes + data */
+		/* create file to hold the data samples */
+		FAudioIOStreamOut *io_data = DumpVoices_fopen(voice, format, "wb", "data");
+		FAudio_close_out(io_data);
+		/* io_data file will be filled by SubmitBuffer */
+	}
+	FAudio_PlatformUnlockMutex((FAudioMutex) io->lock);
+	FAudio_close_out(io);
+}
+
+static void FAudio_DUMPVOICE_Finalize(const FAudioSourceVoice *voice)
+{
+	const FAudioWaveFormatEx *format = voice->src.format;
+
+	/* add dpds subchunk - optional */
+	DumpVoices_finalize_section(voice, format, "dpds");
+	/* add data subchunk */
+	DumpVoices_finalize_section(voice, format, "data");
+
+	/* open main file to update filesize */
+	FAudioIOStreamOut *io = DumpVoices_fopen(voice, format, "r+b", "");
+	if (!io)
+	{
+		return;
+	}
+	FAudio_PlatformLockMutex((FAudioMutex) io->lock);
+	size_t file_size = io->size(io->data);
+	if (file_size >= 44)
+	{
+		/* update filesize */
+		uint32_t chunk_size = (uint32_t)(file_size - 8);
+		io->seek(io->data, 4, FAUDIO_SEEK_SET);
+		io->write(io->data, &chunk_size, 4, 1);
+	}
+	FAudio_PlatformUnlockMutex((FAudioMutex) io->lock);
+	FAudio_close_out(io);
+}
+
+static void FAudio_DUMPVOICE_WriteBuffer(
+	const FAudioSourceVoice *voice,
+	const FAudioBuffer *pBuffer,
+	const FAudioBufferWMA *pBufferWMA,
+	const uint32_t playBegin,
+	const uint32_t playLength
+) {
+	FAudioIOStreamOut *io_data = DumpVoices_fopen(voice, voice->src.format, "ab", "data");
+	if (io_data == NULL)
+	{
+		return;
+	}
+
+	FAudio_PlatformLockMutex((FAudioMutex) io_data->lock);
+	if (pBufferWMA != NULL)
+	{
+		/* dump encoded buffer contents */
+		if (pBufferWMA->PacketCount > 0)
+		{
+			FAudioIOStreamOut *io_dpds = DumpVoices_fopen(voice, voice->src.format, "ab", "dpds");
+			if (io_dpds)
+			{
+				FAudio_PlatformLockMutex((FAudioMutex) io_dpds->lock);
+				/* write to dpds file */
+				io_dpds->write(io_dpds->data, pBufferWMA->pDecodedPacketCumulativeBytes, sizeof(uint32_t), pBufferWMA->PacketCount);
+				FAudio_PlatformUnlockMutex((FAudioMutex) io_dpds->lock);
+				FAudio_close_out(io_dpds);
+			}
+			/* write buffer contents to data file */
+			io_data->write(io_data->data, pBuffer->pAudioData, sizeof(uint8_t), pBuffer->AudioBytes);
+		}
+	}
+	else
+	{
+		/* dump unencoded buffer contents */
+		uint16_t bytesPerFrame = (voice->src.format->nChannels * voice->src.format->wBitsPerSample / 8);
+		FAudio_assert(bytesPerFrame > 0);
+		const void *pAudioDataBegin = pBuffer->pAudioData + playBegin*bytesPerFrame;
+		io_data->write(io_data->data, pAudioDataBegin, bytesPerFrame, playLength);
+	}
+	FAudio_PlatformUnlockMutex((FAudioMutex) io_data->lock);
+	FAudio_close_out(io_data);
+}
+
+#endif /* FAUDIO_DUMP_VOICES */
+
 /* vim: set noexpandtab shiftwidth=8 tabstop=8: */
