@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2018 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2020 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -128,7 +128,7 @@
 		static uint8_t logged = 0; \
 		if (!(condition) && !logged) \
 		{ \
-			SDL_Log("Assertion failed: %s\n", #condition); \
+			SDL_Log("Assertion failed: %s", #condition); \
 			logged = 1; \
 		} \
 	}
@@ -137,7 +137,7 @@
 #endif
 #define FAudio_snprintf SDL_snprintf
 #define FAudio_vsnprintf SDL_vsnprintf
-#define FAudio_Log(msg) SDL_Log("%s\n", msg)
+#define FAudio_Log(msg) SDL_Log("%s", msg)
 #define FAudio_getenv SDL_getenv
 #define FAudio_PRIu64 SDL_PRIu64
 #define FAudio_PRIx64 SDL_PRIx64
@@ -343,6 +343,7 @@ struct FAudio
 	uint8_t version;
 	uint8_t active;
 	uint32_t refcount;
+	uint32_t initFlags;
 	uint32_t updateSize;
 	FAudioMasteringVoice *master;
 	LinkedList *sources;
@@ -352,7 +353,7 @@ struct FAudio
 	FAudioMutex submixLock;
 	FAudioMutex callbackLock;
 	FAudioMutex operationLock;
-	FAudioWaveFormatExtensible *mixFormat;
+	FAudioWaveFormatExtensible mixFormat;
 
 	FAudio_OPERATIONSET_Operation *queuedOperations;
 	FAudio_OPERATIONSET_Operation *committedOperations;
@@ -498,6 +499,10 @@ void FAudio_INTERNAL_AllocEffectChain(
 	const FAudioEffectChain *pEffectChain
 );
 void FAudio_INTERNAL_FreeEffectChain(FAudioVoice *voice);
+uint32_t FAudio_INTERNAL_VoiceOutputFrequency(
+	FAudioVoice *voice,
+	const FAudioVoiceSends *pSendList
+);
 extern const float FAUDIO_INTERNAL_MATRIX_DEFAULTS[8][8][64];
 
 /* Debug */
@@ -713,8 +718,15 @@ void FAudio_FFMPEG_reset(FAudioSourceVoice *voice);
 
 void FAudio_PlatformAddRef(void);
 void FAudio_PlatformRelease(void);
-void FAudio_PlatformInit(FAudio *audio, uint32_t deviceIndex);
-void FAudio_PlatformQuit(FAudio *audio);
+void FAudio_PlatformInit(
+	FAudio *audio,
+	uint32_t flags,
+	uint32_t deviceIndex,
+	FAudioWaveFormatExtensible *mixFormat,
+	uint32_t *updateSize,
+	void** platformDevice
+);
+void FAudio_PlatformQuit(void* platformDevice);
 
 uint32_t FAudio_PlatformGetDeviceCount(void);
 uint32_t FAudio_PlatformGetDeviceDetails(
@@ -742,6 +754,45 @@ void FAudio_sleep(uint32_t ms);
 
 uint32_t FAudio_timems(void);
 
+/* WaveFormatExtensible Helpers */
+
+static inline uint32_t GetMask(uint16_t channels)
+{
+	if (channels == 1) return SPEAKER_MONO;
+	if (channels == 2) return SPEAKER_STEREO;
+	if (channels == 3) return SPEAKER_2POINT1;
+	if (channels == 4) return SPEAKER_QUAD;
+	if (channels == 5) return SPEAKER_4POINT1;
+	if (channels == 6) return SPEAKER_5POINT1;
+	if (channels == 8) return SPEAKER_7POINT1;
+	FAudio_assert(0 && "Unrecognized speaker layout!");
+	return 0;
+}
+
+static inline void WriteWaveFormatExtensible(
+	FAudioWaveFormatExtensible *fmt,
+	int channels,
+	int samplerate
+) {
+	FAudio_assert(fmt != NULL);
+	fmt->Format.wBitsPerSample = 32;
+	fmt->Format.wFormatTag = FAUDIO_FORMAT_EXTENSIBLE;
+	fmt->Format.nChannels = channels;
+	fmt->Format.nSamplesPerSec = samplerate;
+	fmt->Format.nBlockAlign = (
+		fmt->Format.nChannels *
+		(fmt->Format.wBitsPerSample / 8)
+	);
+	fmt->Format.nAvgBytesPerSec = (
+		fmt->Format.nSamplesPerSec *
+		fmt->Format.nBlockAlign
+	);
+	fmt->Format.cbSize = sizeof(FAudioWaveFormatExtensible) - sizeof(FAudioWaveFormatEx);
+	fmt->Samples.wValidBitsPerSample = 32;
+	fmt->dwChannelMask = GetMask(fmt->Format.nChannels);
+	FAudio_memcpy(&fmt->SubFormat, &DATAFORMAT_SUBTYPE_IEEE_FLOAT, sizeof(FAudioGUID));
+}
+
 /* Resampling */
 
 /* Okay, so here's what all this fixed-point goo is for:
@@ -797,4 +848,30 @@ uint32_t FAudio_timems(void);
 	((fxd & FIXED_FRACTION_MASK) * (1.0f / FIXED_ONE)) /* Fraction part */ \
 )
 
+#ifdef FAUDIO_DUMP_VOICES
+/* File writing structure */
+typedef size_t (FAUDIOCALL * FAudio_writefunc)(
+	void *data,
+	const void *src,
+	size_t size,
+	size_t count
+);
+typedef size_t (FAUDIOCALL * FAudio_sizefunc)(
+	void *data
+);
+typedef struct FAudioIOStreamOut
+{
+	void *data;
+	FAudio_readfunc read;
+	FAudio_writefunc write;
+	FAudio_seekfunc seek;
+	FAudio_sizefunc size;
+	FAudio_closefunc close;
+	void *lock;
+} FAudioIOStreamOut;
+
+FAudioIOStreamOut* FAudio_fopen_out(const char *path, const char *mode);
+void FAudio_close_out(FAudioIOStreamOut *io);
+#endif /* FAUDIO_DUMP_VOICES */
+
 /* vim: set noexpandtab shiftwidth=8 tabstop=8: */
