@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2018 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2020 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -84,9 +84,21 @@ void FAudio_PlatformInit(
 	want.format = AUDIO_F32;
 	want.channels = mixFormat->Format.nChannels;
 	want.silence = 0;
-	want.samples = 1024;
 	want.callback = FAudio_INTERNAL_MixCallback;
 	want.userdata = audio;
+	if (flags & FAUDIO_1024_QUANTUM)
+	{
+		/* Get the sample count for a 21.33ms frame.
+		 * For 48KHz this should be 1024.
+		 */
+		want.samples = (int) (
+			want.freq / (1000.0 / (64.0 / 3.0))
+		);
+	}
+	else
+	{
+		want.samples = want.freq / 100;
+	}
 
 	/* Open the device (or at least try to) */
 iosretry:
@@ -95,16 +107,7 @@ iosretry:
 		0,
 		&want,
 		&have,
-#if SDL_VERSION_ATLEAST(2, 0, 9)
-		(flags & FAUDIO_1024_QUANTUM) ? 0 : SDL_AUDIO_ALLOW_SAMPLES_CHANGE
-#else
-#ifdef _WIN32
-#error Windows absolutely positively needs SDL 2.0.9!
-#else
-#pragma message("Please update to SDL 2.0.9 ASAP!")
-#endif
 		0
-#endif
 	);
 	if (device == 0)
 	{
@@ -282,7 +285,7 @@ uint32_t FAudio_timems()
 
 FAudioIOStream* FAudio_fopen(const char *path)
 {
-	FAudioIOStream *io = (FAudioIOStream*) SDL_malloc(
+	FAudioIOStream *io = (FAudioIOStream*) FAudio_malloc(
 		sizeof(FAudioIOStream)
 	);
 	SDL_RWops *rwops = SDL_RWFromFile(path, "rb");
@@ -322,6 +325,31 @@ void FAudio_close(FAudioIOStream *io)
 	FAudio_free(io);
 }
 
+#ifdef FAUDIO_DUMP_VOICES
+FAudioIOStreamOut* FAudio_fopen_out(const char *path, const char *mode)
+{
+	FAudioIOStreamOut *io = (FAudioIOStreamOut*) FAudio_malloc(
+		sizeof(FAudioIOStreamOut)
+	);
+	SDL_RWops *rwops = SDL_RWFromFile(path, mode);
+	io->data = rwops;
+	io->read = (FAudio_readfunc) rwops->read;
+	io->write = (FAudio_writefunc) rwops->write;
+	io->seek = (FAudio_seekfunc) rwops->seek;
+	io->size = (FAudio_sizefunc) rwops->size;
+	io->close = (FAudio_closefunc) rwops->close;
+	io->lock = FAudio_PlatformCreateMutex();
+	return io;
+}
+
+void FAudio_close_out(FAudioIOStreamOut *io)
+{
+	io->close(io->data);
+	FAudio_PlatformDestroyMutex((FAudioMutex) io->lock);
+	FAudio_free(io);
+}
+#endif /* FAUDIO_DUMP_VOICES */
+
 /* UTF8->UTF16 Conversion, taken from PhysicsFS */
 
 #define UNICODE_BOGUS_CHAR_VALUE 0xFFFFFFFF
